/*=============================================================================
	Halfassed time manager, used to correct game tick.
	By Higor, feel free to use this code on your project.
=============================================================================*/

#include "XC_Engine.h"
#include "UnXC_TimeManager.h"


FXC_TimeManager::FXC_TimeManager()
:	MSecAccumulator(0)
,	MSecInterval(100)
,	Factor(1)
,	DeltaAcc(0)
,	IgnoreTimers(10)
{
	appMemzero( &SystemDigest[0], 80); //Size of both arrays

	INT Year;
	INT Month;
	INT DayOfWeek;
	INT Day;
	INT Hour;
	INT Min;
	appSystemTime( Year, Month, DayOfWeek, Day, Hour, Min, LastSec, LastMSec );
}

UBOOL FXC_TimeManager::IsTyped( const TCHAR* Type)
{
	return appStricmp( Type, TEXT("TimeManager")) == 0;
}

INT FXC_TimeManager::Tick( FLOAT DeltaSeconds)
{
	INT Year;
	INT Month;
	INT DayOfWeek;
	INT Day;
	INT Hour;
	INT Min;
	INT Sec;
	INT MSec;
	INT i=0;

	appSystemTime( Year, Month, DayOfWeek, Day, Hour, Min, Sec, MSec );

	if ( Sec < LastSec )
		LastSec -= 60;

	i = MSec - LastMSec;
	if ( (Sec - LastSec) % 60 )
		i += ((Sec - LastSec) % 60) * 1000;
	MSecAccumulator += i;
	DeltaAcc += DeltaSeconds;
	if ( MSecAccumulator >= MSecInterval ) //DO THE MATH HERE
	{
//		if ( MSecAccumulator >= MSecInterval * 2 )
//			debugf( NAME_Log, TEXT("MSEC: SYSTEM RESOURCES HOGGED") );
	
		appMemmove( &SystemDigest[1], &SystemDigest[0], 36 );
		appMemmove( &TimeDigest[1], &TimeDigest[0], 36 );
		SystemDigest[0] = MSec;
		while ( SystemDigest[0] < SystemDigest[1] )
			SystemDigest[0] += 1000;
		TimeDigest[0] = TimeDigest[1] + DeltaAcc;
		for ( i=0 ; i<10 ; i++ )
			TimeDigest[i] -= TimeDigest[9];

		if ( SystemDigest[9] > 1000 )
			for ( i=0 ; i<10 ; i++ )
				SystemDigest[i] -= 1000;

//		INT AvgMSec = (SystemDigest[0] - SystemDigest[9]) / 9;
		INT DevMSec = 0;
		for ( i=1 ; i<10 ; i++ )
			DevMSec += Abs(((SystemDigest[9 - i] - SystemDigest[9]) / i) - MSecInterval);

		if ( DevMSec < MSecInterval )
		{
			FLOAT Delta = TimeDigest[0] * 1000;
//			debugf( NAME_Log, TEXT("MSEC TIMER %i %i %i %f"), MSecAccumulator, AvgMSec, DevMSec, Delta );
			FLOAT fMSecInv = _Reciprocal( (FLOAT)(SystemDigest[0] - SystemDigest[9]) );
			if ( IgnoreTimers <= 0 ) //Don't change factor after ignore timers expire
				Factor = _Reciprocal(Delta * fMSecInv);
			else
				IgnoreTimers--;
		}
//		else
//			debugf( NAME_Log, TEXT("MSEC: DEVIATION TOO HIGH  %i %i %i"), MSecAccumulator, AvgMSec, DevMSec );

		MSecAccumulator = MSecAccumulator % MSecInterval;
		DeltaAcc = 0;
	}
	
	LastMSec = MSec;
	LastSec = Sec;
	return 1;
}



UBOOL FXC_TimeManager::Exec( const TCHAR* Cmd, FOutputDevice& Ar )
{
	guard(FXC_TimeManager::Exec);
	if ( ParseCommand(&Cmd,TEXT("TimeFactor")) )
		Ar.Log( * FString::Printf( TEXT("XC_TimeManager time scaler is set at Factor=%f"), Factor) );
	else
		return 0;
	return 1;
	unguard;
}